/**
 *#########################################################################
 *
 * A component of the Gatherer application, part of the Greenstone digital
 * library suite from the New Zealand Digital Library Project at the
 * University of Waikato, New Zealand.
 *
 * <BR><BR>
 *
 * Author: John Thompson, Greenstone Digital Library, University of Waikato
 *
 * <BR><BR>
 *
 * Copyright (C) 1999 New Zealand Digital Library Project
 *
 * <BR><BR>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * <BR><BR>
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * <BR><BR>
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *########################################################################
 */
package org.greenstone.gatherer.gui;

import calpa.html.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.net.*;
import javax.swing.*;
import org.greenstone.gatherer.Dictionary;
import org.greenstone.gatherer.Gatherer;
import org.greenstone.gatherer.util.GURL;
import org.greenstone.gatherer.util.Utility;
/** This class provides browser-like functionality, with all the standard buttons. The only tricky bit is that the bookmarks are collection specific.
 * @author John Thompson, Greenstone Digital Library, University of Waikato
 * @version 2.3
 */
public class BrowsingPane
    extends JPanel {
    private Bookmarks bookmarks = null;
    private CalHTMLPane view_pane = null;
    private CalHTMLPreferences preferences = null;
    private int back_count = 0;
    private int forward_count = 0;
    private JButton back_button = null;
    private JButton bookmarks_button = null; // Collection bookmarks.
    private JButton forward_button = null;
    private JButton go_button = null;
    private JButton home_button = null;
    private JButton refresh_button = null;
    private JButton reload_button = null;
    private JButton stop_button = null;
    private JLabel status_label = null;
    private JTextField address_field = null;
    private Observer observer = null;
    private String args[] = null;
    static final String BOOKMARKS_FILE = "bookmarks.txt";

    public BrowsingPane() {
	super();

	CalHTMLManager.setCacheDocuments(false);

	// Initialization
	bookmarks = new Bookmarks(); // Empty until collection loaded.
	observer = new Observer();
	preferences = new CalHTMLPreferences();
	preferences.setHomeURL((new GURL(Gatherer.config.getString("general.home_page", false))).getURL());
	// Creation
	// Controls
	JPanel button_pane = new JPanel();
	JPanel left_button_pane = new JPanel();

	back_button = new JButton(Utility.getImage("back.gif"));
	back_button.setEnabled(false);
	Dictionary.registerTooltip(back_button, "Browser.Back_Tooltip");

	// refresh_button = new JButton(Utility.getImage("reload.gif"));
	// refresh_button.setEnabled(false);
	// Dictionary.registerTooltip(refresh_button, "Browser.Reload");

	bookmarks_button = new JButton(Utility.getImage("bookmark.gif"));
	bookmarks_button.setEnabled(false);
	Dictionary.registerTooltip(bookmarks_button, "Browser.Bookmarks");

	home_button = new JButton(Utility.getImage("home.gif"));
	Dictionary.registerTooltip(home_button, "Browser.Home_Tooltip");

	JPanel address_pane = new JPanel();
	address_field = new JTextField(Gatherer.config.getString("general.home_page", false));
	Dictionary.registerTooltip(address_field, "Browser.Address_Tooltip");

	JPanel right_button_pane = new JPanel();
	go_button = new JButton(Utility.getImage("go.gif"));
	Dictionary.registerTooltip(go_button, "Browser.Go_Tooltip");

	stop_button = new JButton(Utility.getImage("stop.gif"));
	stop_button.setEnabled(false);
	Dictionary.registerTooltip(stop_button, "Browser.Stop_Tooltip");

	forward_button = new JButton(Utility.getImage("forward.gif"));
	forward_button.setEnabled(false);
	Dictionary.registerTooltip(forward_button, "Browser.Forward_Tooltip");

	// Calpa Pane stuff
	view_pane = new CalHTMLPane(preferences, observer, "VIEW_PANE");

	// Status bar
	status_label = new JLabel();
	Dictionary.registerText(status_label, "Browser.Ready");

	// Connection
	address_field.addActionListener(new AddressFieldListener());
	back_button.addActionListener(new BackButtonListener());
	bookmarks_button.addActionListener(new BookMarksButtonListener());
	forward_button.addActionListener(new ForwardButtonListener());
	go_button.addActionListener(new GoButtonListener());
	home_button.addActionListener(new HomeButtonListener());
	// refresh_button.addActionListener(new RefreshButtonListener());
	stop_button.addActionListener(new StopButtonListener());

	// Layout
	left_button_pane.setLayout(new GridLayout(1,3));
	left_button_pane.add(back_button);
	left_button_pane.add(bookmarks_button);
	left_button_pane.add(home_button);

	address_pane.setBorder(BorderFactory.createEmptyBorder(10,5,10,5));
	address_pane.setLayout(new BorderLayout());
	address_pane.add(address_field, BorderLayout.CENTER);

	right_button_pane.setLayout(new GridLayout(1,3));
	right_button_pane.add(go_button);
	right_button_pane.add(stop_button);
	right_button_pane.add(forward_button);

	button_pane.setBorder(BorderFactory.createEmptyBorder(5,5,5,5));
	button_pane.setLayout(new BorderLayout());
	button_pane.add(left_button_pane, BorderLayout.WEST);
	button_pane.add(address_pane, BorderLayout.CENTER);
	button_pane.add(right_button_pane, BorderLayout.EAST);

	setLayout(new BorderLayout());
	add(button_pane, BorderLayout.NORTH);
	add(new JScrollPane(view_pane), BorderLayout.CENTER);
	add(status_label, BorderLayout.SOUTH);
    }

    /** Some actions, such as getting the HTML to render properly without throwing sixty bazillion NPE's, can only occur after the frame has been drawn to screen. To this end we have this method so we can action things -after- the frame draw.
     */
    public void afterDisplay() {
	view_pane.goHome();
    }

    /** Called when a significant change occurs to the currently loaded collection.
     * @param ready <i>true</i> if there is a collection ready to be accessed.
     */
    public void collectionChanged(boolean ready) {
	// Reload the bookmarks associated with this collection.
	if (ready) {
	    bookmarks.load();
	}
	else {
	    bookmarks.clear();
	}
    }

    /** If some external class has possibly changed the state of the browsing pane then the controls should be repolled to determine which are still valid.
     */
    public void controlsChanged() {
    }

    /** Retrieves the current address entered in the address Textfield.
     * @return A <strong>String</strong> representing the currently entered address.
     */
    public String getCurrentURL() {
	return address_field.getText();
    }


    /** Called to validate the status of the controls. Disables those that are no longer applicable or restores those that have come back into scope of usage.
     * @param update_address <i>true</i> if this pollControls call should update the address bar, <i>false</i> otherwise.
     */
    private void validateControls() {
	back_button.setEnabled(back_count > 0);
	forward_button.setEnabled(forward_count > 0);
    }

    private class AddressFieldListener
	implements ActionListener {
	public void actionPerformed(ActionEvent event) {
	    // Attempt to create a GURL
	    GURL url = new GURL(address_field.getText());
	    address_field.setText(url.getURL().toString());
	    if(url.valid()) {
		// Now load document
		view_pane.showHTMLDocument(url.getURL());					 
		back_count++;
		forward_count = 0;
	    }
	    validateControls();
	}
    }
	 
    private class BackButtonListener 
	implements ActionListener {
	public void actionPerformed(ActionEvent event) {
	    view_pane.goBack();
	    back_count--;
	    forward_count++;
	    validateControls();
	}
    }

    private class Bookmarks
	extends JDialog {

	private DefaultListModel data = null;

	public Bookmarks() {
	    data = new DefaultListModel();
	}

	public void clear() {
	    data.removeAllElements();
	    // Re-add the "Add Bookmark" entry.
	    //add
	}

	public void load() {
	    // Clear existing
	    data.removeAllElements();
	    // Determine the file to load from...
	    File bookmarks = new File(Gatherer.c_man.getCollectionDirectory() + BOOKMARKS_FILE);
	    if(bookmarks.exists()) {
		// Read in file line at a time adding to bookmarks.
	    }
	}

	public int count() {
	    return data.size();
	}

	private class Entry {
	    public String title = null;
	    public URL url = null;
	    public Entry(String title, String url_raw) {
		this.title = title;
		try {
		    url = new URL(url_raw);
		}
		catch(Exception error) {

		}
	    }
	    public String toString() {
		return title;
	    }
	}
    }

    private class BookMarksButtonListener
	implements ActionListener {
	public void actionPerformed(ActionEvent event) {
				
	}
    }
	 
    private class ForwardButtonListener
	implements ActionListener {
	public void actionPerformed(ActionEvent event) {
	    view_pane.goForward();
	    back_count++;
	    forward_count--;
	    validateControls();
	}
    }

    private class GoButtonListener
	implements ActionListener {
	public void actionPerformed(ActionEvent event) {
	    // Attempt to create a GURL
	    GURL url = new GURL(address_field.getText());
	    address_field.setText(url.getURL().toString());
	    if(url.valid()) {
		// Now load document
		view_pane.showHTMLDocument(url.getURL());	
		back_count++;
		forward_count = 0;
	    }
	    validateControls();
	}
    }
	 
    private class HomeButtonListener
	implements ActionListener {
	public void actionPerformed(ActionEvent event) {
	    address_field.setText(Gatherer.config.getString("general.home_page", false));
	    view_pane.goHome();
	    back_count++;
	    forward_count = 0;
	    validateControls();
	}
    }
	 
    private class RefreshButtonListener
	implements ActionListener {
	public void actionPerformed(ActionEvent event) {
	    view_pane.reloadDocument();
	    validateControls();
	}
    }

    private class StopButtonListener
	implements ActionListener {
	public void actionPerformed(ActionEvent event) {
	    view_pane.stopAll();
	    validateControls();
	}
    }

    private class Observer
	extends DefaultCalHTMLObserver {

	/** Notification that a form submission has been initiated. */
	public void formSubmitUpdate(CalHTMLPane pane, URL docBaseURL, int method, String action, String data) {
	    ///ystem.err.println("Form submitted:" + docBaseURL.toString());
	    ///ystem.err.println("Data: " + data);
	}
          

	public void linkActivatedUpdate(CalHTMLPane pane, URL url, String target_frame, String j_name) {
	    ///ystem.err.println("Link clicked: " + url.toString());
	    address_field.setText(url.toString());
	    back_count++;
	    validateControls();
	}

	public void linkFocusUpdate(CalHTMLPane pane, URL url) {
	    ///ystem.err.println("Link in focus " + url.toString());
	    args = new String[1];
	    args[0] = url.toString();
	    Dictionary.registerText(status_label, "Browser.Follow");
	    args = null;
	}
	public void statusUpdate(CalHTMLPane pane, int state, URL url, int value, String message) {
	    switch(state) {
		// The Pane is attempting to connect to the given URL to receive data.
	    case CalCons.PRE_CONNECT:
		///ystem.err.println("Preconnect: " + url.toString());
		break;
		// The Pane was unable to connect to the given URL or was unable to parse the content. Most likely this will be due to an incorrectly specified URL. The message argument may contain further details of the reason for failure.
	    case CalCons.PARSE_FAILED:
		///ystem.err.println("Parse Failed: " + url.toString());
	        ///ystem.err.println("Message: " + message);
		break;
		// The Pane has established a connection to the given URL and is receiving any content.
	    case CalCons.CONNECTED:
		// Can only stop if somethings happening.
		stop_button.setEnabled(true);
		args = new String[1];
		args[0] = url.toString();
		Dictionary.registerText(status_label, "Browser.Loading", args);
		args = null;
		break;
		// The size of the content at the given URL is known and is contained in the value argument.
	    case CalCons.DOC_LENGTH:
		break;
		// The title of the document for the given URL is known and is contained in the message argument. Only the title of a document in the Pane's top level frame will be sent to this method. If the document has no name, the message will be null, unless the document is a Frameset document in which case the message "Frameset" will be sent.
	    case CalCons.TITLE:
		break;
		// An exception has been thrown during parsing of the data from the URL. This will most likely be an IOException such as a server time-out.
	    case CalCons.PARSE_FAILED_POST_CONNECT:
		///ystem.err.println("Parse Failed Post Connect: " + url.toString());
		break;
		// The document has been parsed but formatting cannot be completed because the document contains images of unspecified size. The parsing thread is waiting for image updates to give it the information it needs to format and display the document.
	    case CalCons.WAITING_FOR_IMAGES:
		break;
		// All text and image data has been received, parsed and the document structure determined.
	    case CalCons.DOC_LOADED:
		// All done.
		stop_button.setEnabled(false);
		// Rendering complete.
		Dictionary.registerText(status_label, "Browser.Ready");
		go_button.setEnabled(true);
		stop_button.setEnabled(false);
		// refresh_button.setEnabled(true);
		// Update mirror pane
		Gatherer.g_man.mirror_pane.setURL(url.toString());
		break;
	    }
	}
    }
}
